@@ -4,7 +4,7 @@ use warnings;
 
 package Git::Hooks;
 {
-  $Git::Hooks::VERSION = '0.001';
+  $Git::Hooks::VERSION = '0.002';
 }
 # ABSTRACT: A framework for implementing Git hooks.
 
@@ -113,6 +113,42 @@ sub get_affected_ref_commits {
     return $affected_refs{$ref}{commits};
 }
 
+# This is an internal routine used to invoke external hooks, feed them
+# the needed input and wait for them.
+
+sub spawn_external_file {
+    my ($file, $hook, @args) = @_;
+
+    my $exit;
+    if ($hook ne 'pre-receive') {
+	$exit = system {$file} ($hook, @args);
+    } else {
+	my $pid = open my $pipe, '|-';
+	if (! defined $pid) {
+	    die __PACKAGE__, ": can't fork: $!\n";
+	} elsif ($pid) {
+	    # parent
+	    foreach my $ref (get_affected_refs()) {
+		my ($old, $new) = get_affected_ref_range($ref);
+		say $pipe "$old $new $ref";
+	    }
+	    $exit = close $pipe;
+	} else {
+	    # child
+	    exec {$file} ($hook, @args);
+	    die __PACKAGE__, ": can't exec: $!\n";
+	}
+    }
+    unless ($exit == 0) {
+	die __PACKAGE__, ": failed to execute '$file': $!\n"
+	    if $? == -1;
+	die sprintf(__PACKAGE__, ": '$file' died with signal %d, %s coredump\n",
+		    ($? & 127), ($? & 128) ? 'with' : 'without')
+	    if $? & 127;
+	die sprintf(__PACKAGE__, ": '$file' exited abnormally with value %d\n", $? >> 8);
+    }
+}
+
 sub run_hook {
     my ($hook_name, @args) = @_;
 
@@ -120,49 +156,58 @@ sub run_hook {
 
     $Git = Git::More->repository();
 
-    # If there's no githooks options we are disabled.
-    my $config = hook_config('githooks') or return;
+    my $config = hook_config('githooks');
 
-    # Grok the enabled hooks. Return if there is none.
-    my $enabled_hooks = $config->{$hook_name} or return;
-
-    # Some hooks affect lists of commits. Let's grok them at once.
+    # Some hooks (pre-receive and update) affect refs and associated
+    # commit ranges. Let's grok them at once.
     grok_affected_refs($hook_name);
 
-    # Grok the directories where we'll look for the hook scripts.
-    my @hooksdir;
-    # First the local directory 'hooks.d' under the repository path
-    push @hooksdir, 'hooks.d';
-    # Then, the optional list of directories specified by the hooksdir
-    # config option
-    push @hooksdir, @{$config->{hooksdir}} if exists $config->{hooksdir};
-    # And finally, the Git::Hooks standard hooks directory
-    push @hooksdir, catfile(dirname($INC{'Git/Hooks.pm'}), 'Hooks');
+    # Invoke enabled plugins
+    if (my $enabled_hooks = $config->{$hook_name}) {
+	# Define the list of directories where we'll look for the hook
+	# plugins. First the local directory 'githooks' under the
+	# repository path, then the optional list of directories
+	# specified by the githooks.plugins config option, and,
+	# finally, the Git::Hooks standard hooks directory.
+	unshift @{$config->{plugins}}, 'githooks';
+	push    @{$config->{plugins}}, catfile(dirname($INC{'Git/Hooks.pm'}), 'Hooks');
 
-    # Execute every enabled script found in the @hooksdir directories
-    foreach my $hook (@$enabled_hooks) {
-	my $found = 0;
-	foreach my $dir (grep {-d} @hooksdir) {
-	    my $script = catfile($dir, $hook);
-	    next unless -f $script;
+	foreach my $hook (@$enabled_hooks) {
+	    my $found = 0;
+	    foreach my $dir (grep {-d} @{$config->{plugins}}) {
+		my $script = catfile($dir, $hook);
+		next unless -f $script;
 
-	    my $exit = do $script;
-	    unless ($exit) {
-		die __PACKAGE__, ": couldn't parse $script: $@\n" if $@;
-		die __PACKAGE__, ": couldn't do $script: $!\n"    unless defined $exit;
-		die __PACKAGE__, ": couldn't run $script\n"       unless $exit;
+		my $exit = do $script;
+		unless ($exit) {
+		    die __PACKAGE__, ": couldn't parse $script: $@\n" if $@;
+		    die __PACKAGE__, ": couldn't do $script: $!\n"    unless defined $exit;
+		    die __PACKAGE__, ": couldn't run $script\n"       unless $exit;
+		}
+		$found = 1;
 	    }
-	    $found = 1;
+	    die __PACKAGE__, ": can't find hook enabled hook $hook.\n" unless $found;
+	}
+
+	# Call every hook function installed by the hook scripts before.
+	foreach my $hook (values %{$Hooks{$hook_name}}) {
+	    $hook->($Git, @args);
 	}
-	die __PACKAGE__, ": can't find hook enabled hook $hook.\n" unless $found;
     }
 
-    # Call every hook function installed by the hook scripts before.
-    foreach my $hook (values %{$Hooks{$hook_name}}) {
-	$hook->($Git, @args);
-    }
+    # Invoked enabled external hooks
+    unless (exists $config->{externals} && ! $config->{externals}[-1]) {
+	# By default, start looking for external hooks in the
+	# '.git/hooks.d' directory.
+	unshift @{$config->{hooks}}, catfile($Git->repo_path(), 'hooks.d');
 
-    return;
+	foreach my $dir (grep {-e} map {catfile($_, $hook_name)} @{$config->{hooks}}) {
+	    opendir my $dh, $dir or die __PACKAGE__, ": cannot opendir $dir: $!\n";
+	    foreach my $file (grep {-f && -x} map {catfile($dir, $_)} readdir $dh) {
+		spawn_external_file($file, $hook_name, @args);
+	    }
+	}
+    }
 }
 
 
@@ -177,7 +222,7 @@ Git::Hooks - A framework for implementing Git hooks.
 
 =head1 VERSION
 
-version 0.001
+version 0.002
 
 =head1 SYNOPSIS
 
@@ -199,11 +244,9 @@ A single script can implement several Git hooks:
 
 	run_hook($0, @ARGV);
 
-Or you can use already implemented hooks which are installed in the
-Git/Hooks directory, in the Git/Hooks.pm installation. These hooks are
-enabled by Git configuration options. (More on this later.) If you're
-only using Git::Hooks standard hooks, all you need is the following
-script:
+Or you can use Git::Hooks plugins or external hooks, driven by the
+single script below. These hooks are enabled by Git configuration
+options. (More on this later.)
 
 	#!/usr/bin/env perl
 
@@ -211,7 +254,7 @@ script:
 
 	run_hook($0, @ARGV);
 
-=for Pod::Coverage applypatch_msg pre_applypatch post_applypatch pre_commit prepare_commit_msg commit_msg post_commit pre_rebase post_checkout post_merge pre_receive update post_receive post_update pre_auto_gc post_rewrite grok_affected_refs
+=for Pod::Coverage grok_affected_refs spawn_external_file
 
 =head1 INTRODUCTION
 
@@ -272,8 +315,8 @@ command, which spawns yet another process.
 
 =back
 
-Git::Hooks is a framework for implementing Git hooks that tries to
-solve these problems.
+Git::Hooks is a framework for implementing Git and driving existing
+external hooks in a way that tries to solve these problems.
 
 Instead of having separate scripts implementing different
 functionality you may have a single script implementing all the
@@ -283,6 +326,9 @@ namespace. This single script can be used to implement all standard
 hooks, because each hook knows when to perform based on the context in
 which the script was called.
 
+If you already have some handy hooks and want to keep using them,
+don't worry. Git::Hooks can drive external hooks very easily.
+
 =head1 USAGE
 
 Go to the C<.git/hooks> directory under the root of your Git
@@ -311,69 +357,15 @@ for nothing.)
 	$ ln -s git-hooks.pl post-commit
 
 As is, the script won't do anything. You have to implement some hooks
-in it or use some of the existing ones implemented as plugins. Either
-way, the script should end with a call to C<run_hooks> passing to it
-the name with which it was called (C<$0>) and all the arguments it
-received (C<@ARGV>).
+in it, use some of the existing plugins, or set up some external
+plugins to be invoked properly. Either way, the script should end with
+a call to C<run_hooks> passing to it the name with which it was called
+(C<$0>) and all the arguments it received (C<@ARGV>).
 
 =head2 Implementing Hooks
 
-Implement hooks using one of the hook I<directives> below. Each one of
-them gets a single block (anonymous function) as argument. The block
-will be called by C<run_hook> with proper arguments, as indicated
-below. These arguments are the ones gotten from @ARGV, with the
-exception of the ones identified by C<Git>. These are Git::More
-objects which can be used to grok detailed information about the
-repository and the current transaction. (Please, refer to the
-L<Git::More> documentation to know how to use it.)
-
-=over
-
-=item * APPLYPATCH_MSG(Git, commit-msg-file)
-
-=item * PRE_APPLYPATCH(Git)
-
-=item * POST_APPLYPATCH(Git)
-
-=item * PRE_COMMIT(Git)
-
-=item * PREPARE_COMMIT_MSG(Git, commit-msg-file [, msg-src [, SHA1]])
-
-=item * COMMIT_MSG(Git, commit-msg-file)
-
-=item * POST_COMMIT(Git)
-
-=item * PRE_REBASE(Git)
-
-=item * POST_CHECKOUT(Git, prev-head-ref, new-head-ref, is-branch-checkout)
-
-=item * POST_MERGE(Git, is-squash-merge)
-
-=item * PRE_RECEIVE(Git)
-
-=item * UPDATE(Git, updated-ref-name, old-object-name, new-object-name)
-
-=item * POST_RECEIVE(Git)
-
-=item * POST_UPDATE(Git, updated-ref-name, ...)
-
-=item * PRE_AUTO_GC(Git)
-
-=item * POST_REWRITE(Git, command)
-
-=back
-
-Note that the hook directives resemble function definitions but they
-aren't. They are function calls, and as such must end with a
-semi-colon.
-
-Most of the hooks are used to check some condition. If the condition
-holds, they must simply end without returning anything. Otherwise,
-they must C<die> with a suitable error message. This will prevent Git
-from finishing its operation.
-
-Also note that each hook directive can be called more than once if you
-need to implement more than one specific hook. For example:
+Implement hooks using one of the hook I<directives> described in the
+MAIN METHODS section. For example:
 
     # Check if every added/updated file is smaller than a fixed limit.
 
@@ -444,7 +436,40 @@ JIRA issues.
 Each plugin may be used in one or, sometimes, multiple hooks. Their
 documentation is explicit about this.
 
-=head2 Configuration
+=head2 Invoking external hooks
+
+Since the default Git hook scripts are taken by the Git::Hooks driver
+script, you must install your external hooks somewhere else. By
+default, the external plugin will look for external hook scripts in
+the directory C<.git/hooks.d> (which you must create) under the
+repository. Below this directory you should have another level of
+directories, named after the default hook names, under which you can
+drop your external hooks.
+
+For example, let's say you want to use some of the hooks in the
+standard Git package
+(L<https://github.com/gitster/git/blob/b12905140a8239ac687450ad43f18b5f0bcfb62e/contrib/hooks/update-paranoid>). You
+should copy each of those scripts to a file under the appropriate hook
+directory, like this:
+
+=over
+
+=item C<.git/hooks.d/pre-auto-gc/pre-auto-gc-battery>
+
+=item C<.git/hooks.d/pre-commit/setgitperms.perl>
+
+=item C<.git/hooks.d/post-receive/post-receive-email>
+
+=item C<.git/hooks.d/update/update-paranoid>
+
+=back
+
+Note that you may install more than one script under the same
+hook-named directory. The driver will execute all of them in a
+non-specified order. If any of them exits abnormally, the driver will
+exit with an appropriate error message.
+
+=head1 CONFIGURATION
 
 Git::Hooks is configured via Git's own configuration
 infrastructure. The framework defines a few options which are
@@ -460,15 +485,43 @@ repository. Note that this will fetch all C<--system>, C<--global>,
 and C<--local> options, in this order. You may use this mechanism to
 define configuration global to a user or local to a repository.
 
-=over
+=head2 githooks.applypatch-msg     PLUGIN
 
-=item githooks.HOOK
+=head2 githooks.pre-applypatch     PLUGIN
 
-To enable a plugin you must register it with the appropriate
-C<githooks.HOOK> option. For instance, if you want to enable the
-C<check-jira.pl> plugin in the C<update> hook, you must do this:
+=head2 githooks.post-applypatch    PLUGIN
 
-    $ git config githooks.update check-jira.pl
+=head2 githooks.pre-commit         PLUGIN
+
+=head2 githooks.prepare-commit-msg PLUGIN
+
+=head2 githooks.commit-msg  	   PLUGIN
+
+=head2 githooks.post-commit 	   PLUGIN
+
+=head2 githooks.pre-rebase  	   PLUGIN
+
+=head2 githooks.post-checkout      PLUGIN
+
+=head2 githooks.post-merge         PLUGIN
+
+=head2 githooks.pre-receive        PLUGIN
+
+=head2 githooks.update             PLUGIN
+
+=head2 githooks.post-receive       PLUGIN
+
+=head2 githooks.post-update        PLUGIN
+
+=head2 githooks.pre-auto-gc        PLUGIN
+
+=head2 githooks.post-rewrite       PLUGIN
+
+To enable a plugin you must register it with one of the above
+options. For instance, if you want to enable the C<check-jira.pl>
+plugin in the C<update> hook, you must do this:
+
+    $ git config --add githooks.update check-jira.pl
 
 Note that you may enable more than one plugin to the same hook. For
 instance:
@@ -478,17 +531,17 @@ instance:
 And you may enable the same plugin in more than one hook, if it makes
 sense to do so. For instance:
 
-    $ git config githooks.commit-msg check-jira.pl
+    $ git config --add githooks.commit-msg check-jira.pl
 
-=item githooks.hooksdir
+=head2 githooks.plugins DIR
 
 The plugins enabled for a hook are searched for in three places. First
-they're are searched for in the C<hooks.d> directory under the
-repository path (usually in C<.git/hooks.d>), so that you may have
+they're are searched for in the C<githooks> directory under the
+repository path (usually in C<.git/githooks>), so that you may have
 repository specific hooks (or repository specific versions of a hook).
 
 Then, they are searched for in every directory specified with the
-C<githooks.hooksdir> option.  You may set it more than once if you
+C<githooks.plugins> option.  You may set it more than once if you
 have more than one directory holding your hooks.
 
 Finally, they are searched for in Git::Hooks installation.
@@ -498,14 +551,26 @@ the search stops. So, you may want to copy one of the standard plugins
 and change it to suit your needs better. (Don't shy away from sending
 your changes back to us, though.)
 
-=back
+=head2 githooks.externals [01]
+
+By default the driver script will look for external hooks after
+executing every enabled plugins. You may disable external hooks
+invokation by setting this option to 0.
+
+=head2 githooks.hooks DIR
+
+You can tell this plugin to look for external hooks in other
+directories by specifying them with this option. The directories
+specified here will be looked for after the default directory
+C<.git/hooks.d>, so that you can use this option to have some global
+external hooks shared by all of your repositories.
 
 Please, see the plugins documentation to know about their own
 configuration options.
 
-=head1 EXPORT
+=head1 MAIN METHOD
 
-=head2 run_hook NAME ARGS...
+=head2 run_hook(NAME, ARGS...)
 
 This is the main routine responsible to invoke the right hooks
 depending on the context in which it was called.
@@ -519,7 +584,63 @@ called. Usually you just pass C<@ARGV> to it. And that's it. Mostly.
 
 	run_hook($0, @ARGV);
 
-=head1 PLUGIN DEVELOPER TUTORIAL
+=head1 HOOK DIRECTIVES
+
+Hook directives are routines you use to register routines as hooks.
+Each one of the hook directives gets a routine-ref or a single block
+(anonymous routine) as argument. The routine/block will be called by
+C<run_hook> with proper arguments, as indicated below. These arguments
+are the ones gotten from @ARGV, with the exception of the ones
+identified by GIT. These are C<Git::More> objects which can be used to
+grok detailed information about the repository and the current
+transaction. (Please, refer to the L<Git::More> documentation to know
+how to use them.)
+
+Note that the hook directives resemble function definitions but they
+aren't. They are function calls, and as such must end with a
+semi-colon.
+
+Most of the hooks are used to check some condition. If the condition
+holds, they must simply end without returning anything. Otherwise,
+they must C<die> with a suitable error message. On some hooks, this
+will prevent Git from finishing its operation.
+
+Also note that each hook directive can be called more than once if you
+need to implement more than one specific hook.
+
+=head2 APPLYPATCH_MSG(GIT, commit-msg-file)
+
+=head2 PRE_APPLYPATCH(GIT)
+
+=head2 POST_APPLYPATCH(GIT)
+
+=head2 PRE_COMMIT(GIT)
+
+=head2 PREPARE_COMMIT_MSG(GIT, commit-msg-file [, msg-src [, SHA1]])
+
+=head2 COMMIT_MSG(GIT, commit-msg-file)
+
+=head2 POST_COMMIT(GIT)
+
+=head2 PRE_REBASE(GIT)
+
+=head2 POST_CHECKOUT(GIT, prev-head-ref, new-head-ref, is-branch-checkout)
+
+=head2 POST_MERGE(GIT, is-squash-merge)
+
+=head2 PRE_RECEIVE(GIT)
+
+=head2 UPDATE(GIT, updated-ref-name, old-object-name, new-object-name)
+
+=head2 POST_RECEIVE(GIT)
+
+=head2 POST_UPDATE(GIT, updated-ref-name, ...)
+
+=head2 PRE_AUTO_GC(GIT)
+
+=head2 POST_REWRITE(GIT, command)
+
+=head1 METHODS FOR PLUGIN DEVELOPERS
 
 Plugins should start by importing the utility routines from
 Git::Hooks:
@@ -527,8 +648,8 @@ Git::Hooks:
     use Git::Hooks qw/:utils/;
 
 Usually at the end, the plugin should use one or more of the hook
-directives defined in the C<Implementing Hooks> section above to
-install its hook routines in the apropriate hooks.
+directives defined above to install its hook routines in the
+apropriate hooks.
 
 Every hook routine receives a Git::More object as its first
 argument. You should use it to infer all needed information from the
@@ -540,14 +661,14 @@ this. Hopefully it's not that hard.
 
 The utility routines implemented by Git::Hooks are the following:
 
-=head2 hook_config NAME
+=head2 hook_config(NAME)
 
 This routine returns a hash-ref containing every configuration
 variable for the section NAME. It's usually called with the name of
 the plugin as argument, meaning that the plugin configuration is
 contained in a section by its name.
 
-=head2 is_ref_enabled SPECS REF
+=head2 is_ref_enabled(SPECS, REF)
 
 This routine returns a boolean indicating if REF matches one of the
 ref-specs in SPECS. REF is the complete name of a Git ref and SPECS is
@@ -562,7 +683,7 @@ Each rule in SPECS may indicate the matching refs as the complete ref
 name (e.g. "refs/heads/master") or by a regular expression starting
 with a caret (C<^>), which is kept as part of the regexp.
 
-=head2 get_affected_refs
+=head2 get_affected_refs()
 
 This routine returns a list of all the Git refs affected by the
 current operation. During the C<update> hook it returns the single ref
@@ -570,29 +691,33 @@ passed via the command line. During the C<pre-receive> hook it returns
 the list of refs passed via STDIN. During any other hook it returns
 the empty list.
 
-=head2 get_affected_ref_range REF
+=head2 get_affected_ref_range(REF)
 
 This routine returns the two-element list of commit ids representing
 the OLDCOMMIT and the NEWCOMMIT of the affected REF.
 
-=head2 get_affected_ref_commit_ids REF
+=head2 get_affected_ref_commit_ids(REF)
 
 This routine returns the list of commit ids leading from the affected
 REF's NEWCOMMIT to OLDCOMMIT.
 
-=head2 get_affected_ref_commits REF
+=head2 get_affected_ref_commits(REF)
 
 This routine returns the list of commits leading from the affected
 REF's NEWCOMMIT to OLDCOMMIT. The commits are represented by hashes,
 as returned by C<Git::More::get_commits>.
 
+=head1 SEE ALSO
+
+C<Git::More>.
+
 =head1 AUTHOR
 
 Gustavo L. de M. Chaves <gnustavo@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2012 by Gustavo L. de M. Chaves.
+This software is copyright (c) 2012 by CPqD <www.cpqd.com.br>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
